cd(@__DIR__)
println("Loading Modules")
using Ipopt         # Solver
using NLOptControl  # Problem definition and discretization
using PrettyPlots   # Plots for optimization problem
using Plots         # Custom plots
include("utils.jl")

println("Setup problem")
# Setup properties ====================================================================
l    = 0.345
lact = 0.4
g    = 9.81
# Initial and final constraints
X0   = [-0.8, 0, 0, 0, 0]
XF   = [-0.3-lact, 1.4, -pi/2*0.84, 0.0, NaN]
# Upper and lower constraints on control and states
CL   = [-100.]
CU   = [100.]
XL   = [-0.9,-Inf,-Inf,-Inf,-5.0]
XU   = [0.1,Inf,Inf,Inf,5.0]

# Setup model equations ================================================================
dx    = Array{Expr}(5)
dx[1] = :(pd[j])
dx[2] = :(ar[j])
dx[3] = :(thetad[j])
dx[4] = :(-$g/$l*sin(theta[j]) - 0.04*thetad[j] + 1/$l*cos(theta[j])*ar[j])
dx[5] = :(ard[j])

# Create problem
n   = define(numStates = 5, numControls=1, X0=X0, XF=XF, CL=CL, CU=CU, XL=XL, XU=XU)
# Give name to states and controls
states!(n,[:p, :pd, :theta, :thetad, :ar],descriptions=["p(t)", "ṗ(t)", "θ(t)", "̇θ(t))", "aᵣ(t)"])
controls!(n,[:ard],descriptions=["ȧᵣ(t)"])
dynamics!(n,dx)

# Configure ============================================================================
N = 800 # Number of grid points in discretization
# Final time free, backward Euler discretization
configure!(n; :finalTimeDV=>true, :integrationScheme=>:bkwEuler, N = N, solverSettings=(:print_level => 3))
# Set objective to final time
@NLobjective(n.ocp.mdl, Min, n.ocp.tf)

srand(1)
setvalue(n.ocp.tf, 2.5)
randomInit!(n, N) # Make some noise !!!!

# Solve ================================================================================
println("Optimizing")
# 5+ minutes
@time optimize!(n)
dfs = Array(n.r.ocp.dfs[1])
#writedlm("initial_guess_part_1.csv", dfs, " ")

# Initial and final constraints
X0   = dfs[end,2:6]
XF   = [0.0, 0, 0, 0, NaN]
# Create problem
n   = define(numStates = 5, numControls=1, X0=X0, XF=XF, CL=CL, CU=CU, XL=XL, XU=XU)
# Give name to states and controls
states!(n,[:p, :pd, :theta, :thetad, :ar],descriptions=["p(t)", "ṗ(t)", "θ(t)", "̇θ(t))", "aᵣ(t)"])
controls!(n,[:ard],descriptions=["ȧᵣ(t)"])
dynamics!(n,dx)


# Configure ============================================================================
N = 600 # Number of grid points in discretization
# Final time free, backward Euler discretization
configure!(n; :finalTimeDV=>true, :integrationScheme=>:bkwEuler, N = N, solverSettings=(:print_level => 3))
# Set objective to final time
@NLobjective(n.ocp.mdl, Min, n.ocp.tf)


srand(1)
setvalue(n.ocp.tf, 1.5)
randomInit!(n, N) # Make some noise

# Solve ================================================================================
println("Optimizing")
# ? minutes
@time optimize!(n)

dfs2 = Array(n.r.ocp.dfs[1]) # Data from solver

dfs2[:,1] .+= dfs[end,1]

total = [dfs;dfs2]


p       = total[:,2]
theta   = total[:,4]
xp      = p .- lact*sin.(theta)
yp      = -lact*cos.(theta)

anim2 = @animate for k = (1:24:size(total,1)+1)
    i = k>size(total,1) ? size(total,1) : k
    plot([-1,1], [0,0], ylims=(-0.8,0.8), xlims=(-1.3,0.1), aspect_ratio=1, lab="") # Beam
    plot!([p[i], p[i]-lact*sin(theta[i])], [0.0, -lact*cos(theta[i])], lab="")      # Pendulum
    scatter!([p[i],], [0.0], m=:star, lab="")                                       # Pendulum base
    plot!(-0.3-0.05*sin.(0:0.1:2pi), -0.4-0.3*cos.(0:0.1:2pi), lab="")              # Constraint
    scatter!([p[1:i]-lact*sin.(theta[1:i])], [-lact*cos.(theta[1:i])], m=(:circle,3.0), lab="") # Mass
end
# Not real time
gif(anim2, "/tmp/pendulum_init_full.gif", fps = round(Int, (size(total,1)/24)/total[end,1]))

writedlm("initial_guess.csv", ["time" "p" "p_dot" "theta" "theta_dot" "a_ref" "a_ref_dot"; total], " ")
